//--------------------------------------------------------------
// File     : stm32_ub_touch_480x272.c
// Datum    : 17.06.2015
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// Board    : STM32F429-Board von UB (Version 1.0)
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, ADC, TIM, MISC
// Funktion : Touch Funktionen (4Wire-Resistiv Touch)
//            Der Zugriff erfolgt ueber 4 GPIO Pins
//
//  Der Touch wird per Timer7 (TIM7) und ADC3 zyklisch abgefragt (50ms)
//  und die Daten werden in einer Struktur gespeichert :
//
//            Touch_Data.xp = X-Position [0...479]
//            Touch_Data.yp = Y-Position [0...271]
//            Touch_Data.status = [TOUCH_PRESSED, TOUCH_RELEASED]
//              -> Koordinaten nur gltig bei "TOUCH_PRESSED"
//
//          
// Hinweis  : Das Touch benutzt die CPU-Pins :
//
//             PA0  = XR (X right)
//             PA1  = YD (Y down)
//             PA2  = XL (X left)
//             PA3  = YU (Y up)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_touch_480x272.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_Touch_480x272_GPIOConfig(uint8_t mode);
void P_Touch_480x272_ADCInit(void);
void P_Touch_480x272_TIMInit(void);
void P_Touch_480x272_ReadTouch(void);
uint8_t P_Touch_480x272_ReadADC(uint8_t mode);
void P_Touch_480x272_Sort(void);
void P_Touch_480x272_Delay(volatile uint32_t nCount);




//--------------------------------------------------------------
// Init vom Touch
// Return_wert :
//  -> ERROR   , wenn Touch nicht gefunden wurde
//  -> SUCCESS , wenn Touch OK
//--------------------------------------------------------------
ErrorStatus UB_Touch_Init(void)
{
  // init aller Variabeln
  Touch_Data.status=TOUCH_RELEASED;
  Touch_Data.xp=0;
  Touch_Data.yp=0;
  
  TOUCH_RAW.x_adcwert=0;
  TOUCH_RAW.y_adcwert=0;
  TOUCH_RAW.timer_ms=0;
  
  // init der GPIOs
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_I);
  // init vom ADC
  P_Touch_480x272_ADCInit();
  // init vom timer
  P_Touch_480x272_TIMInit();

  
  // auf "Sleep"
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_S);

  return(SUCCESS);
}


//--------------------------------------------------------------
// interne Funktion
// einstellen der GPIO Pins
// mode : TOUCH_MODE_I = init
//        TOUCH_MODE_S = sleep
//        TOUCH_MODE_X = messung der x-achse (an YU)
//        TOUCH_MODE_Y = messung der y-achse (an XL)
//--------------------------------------------------------------
void P_Touch_480x272_GPIOConfig(uint8_t mode)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  if(mode==TOUCH_MODE_I) {
    RCC_AHB1PeriphClockCmd(TOUCH_GPIO_CLOCK, ENABLE);         
  }
  else if(mode==TOUCH_MODE_S) {
    // set pin XR as output (pushpull) and GND
    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XR;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_XR, &GPIO_InitStructure);    
    TOUCH_GPIO_PORT_XR->BSRRH = TOUCH_GPIO_PIN_XR; 
    // set pin XL as output (pushpull) and VCC
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XL;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_XL, &GPIO_InitStructure);          
    TOUCH_GPIO_PORT_XL->BSRRL = TOUCH_GPIO_PIN_XL;  
    // set pin YD as output (pushpull) and GND
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YD;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_YD, &GPIO_InitStructure);     
    TOUCH_GPIO_PORT_YD->BSRRH = TOUCH_GPIO_PIN_YD;  
    // set pin YU as output (pushpull) and VCC
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YU;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_YU, &GPIO_InitStructure);
    TOUCH_GPIO_PORT_YU->BSRRL = TOUCH_GPIO_PIN_YU;    
  }
  else if(mode==TOUCH_MODE_X) {
    // set pin XR as output (pushpull) and GND
    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XR;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_XR, &GPIO_InitStructure);    
    TOUCH_GPIO_PORT_XR->BSRRH = TOUCH_GPIO_PIN_XR; 
    // set pin XL as output (pushpull) and VCC
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XL;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_XL, &GPIO_InitStructure);          
    TOUCH_GPIO_PORT_XL->BSRRL = TOUCH_GPIO_PIN_XL; 
    // set pin YD as input (PullDown)
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YD;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;        
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_Init(TOUCH_GPIO_PORT_YD, &GPIO_InitStructure);  
    // set pin YU as adc-input (PullUp)
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YU;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;     
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(TOUCH_GPIO_PORT_YU, &GPIO_InitStructure);    
  }
  else if(mode==TOUCH_MODE_Y) {
    // set pin XR as input (PullDown)
    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XR;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;      
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;  
    GPIO_Init(TOUCH_GPIO_PORT_XR, &GPIO_InitStructure);   
    // set pin XL as adc-input (PullUp)
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_XL;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;      
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(TOUCH_GPIO_PORT_XL, &GPIO_InitStructure);     
    // set pin YD as output (pushpull) and GND
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YD;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_YD, &GPIO_InitStructure);
    TOUCH_GPIO_PORT_YD->BSRRH = TOUCH_GPIO_PIN_YD;   
    // set pin YU as output (pushpull) and VCC
    GPIO_InitStructure.GPIO_Pin = TOUCH_GPIO_PIN_YU;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(TOUCH_GPIO_PORT_YU, &GPIO_InitStructure);
    TOUCH_GPIO_PORT_YU->BSRRL = TOUCH_GPIO_PIN_YU;     
  }
}

//--------------------------------------------------------------
// interne Funktion
// init vom ADC
//--------------------------------------------------------------
void P_Touch_480x272_ADCInit(void)
{
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  ADC_InitTypeDef       ADC_InitStructure; 
  
  // clock enable  
  RCC_APB2PeriphClockCmd(TOUCH_ADC_CLOCK, ENABLE);

  // ADC-Config
  ADC_CommonStructInit(&ADC_CommonInitStructure);
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div6;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_10Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);
  
  ADC_StructInit(&ADC_InitStructure);
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_10b;
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;
  ADC_Init(TOUCH_ADC_NAME, &ADC_InitStructure);
  
  // ADC-Enable
  ADC_Cmd(TOUCH_ADC_NAME, ENABLE);   
}

//----------------------------------------------------------------------
// interne Funktion
// init vom Timer
//----------------------------------------------------------------------
void P_Touch_480x272_TIMInit(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  // clock enable
  RCC_APB1PeriphClockCmd(TOUCH_TIMER_CLOCK, ENABLE);

  // set period + prescaler
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Period =  TOUCH_TIMER_PERIOD;
  TIM_TimeBaseStructure.TIM_Prescaler = TOUCH_TIMER_PRESCALE;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TOUCH_TIMER_NAME, &TIM_TimeBaseStructure);

  // Timer enable
  TIM_Cmd(TOUCH_TIMER_NAME, ENABLE);

  // TIM Update Interrupt enable
  TIM_ITConfig(TOUCH_TIMER_NAME,TIM_IT_Update,ENABLE);

  // NVIC konfig
  NVIC_InitStructure.NVIC_IRQChannel = TOUCH_TIMER_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

//--------------------------------------------------------------
// interne Funktion
// auslesen vom Touch
// (diese Funktion wird zyklisch vom Timer aufgerufen)
//
// Touch_Data.status : [TOUCH_PRESSED, TOUCH_RELEASED]
// Touch_Data.xp     : [0...479]
// Touch_Data.yp     : [0...271]
//--------------------------------------------------------------
void P_Touch_480x272_ReadTouch(void)
{
  uint8_t c1=1,c2=1;
  volatile float istwert;
  uint16_t x1,y1;

  // X-Achse
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_X);
  c1=P_Touch_480x272_ReadADC(TOUCH_MODE_X);
  if(c1==0) x1=TOUCH_RAW.x_adcwert;
  // kurz auf sleep (wichtig)
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_S);
  // Y-Achse
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_Y);
  c2=P_Touch_480x272_ReadADC(TOUCH_MODE_Y);
  if(c2==0) y1=TOUCH_RAW.y_adcwert;
  // kurz auf sleep (wichtig)
  P_Touch_480x272_GPIOConfig(TOUCH_MODE_S);

  // auswertung
  if((c1==0) && (c2==0)) {

    // x-koordinate umrechnen
    istwert=x1;
    #if TOUCH_ADC_X0>TOUCH_ADC_X480
      #define   TOUCH_ADC_XDELTA      TOUCH_ADC_X0-TOUCH_ADC_X480
      if(istwert<TOUCH_ADC_X480) istwert=TOUCH_ADC_X480;
      if(istwert>TOUCH_ADC_X0) istwert=TOUCH_ADC_X0;
      istwert=TOUCH_ADC_X0-istwert;
    #else
      #define   TOUCH_ADC_XDELTA      TOUCH_ADC_X480-TOUCH_ADC_X0
      if(istwert<TOUCH_ADC_X0) istwert=TOUCH_ADC_X0;
      if(istwert>TOUCH_ADC_X480) istwert=TOUCH_ADC_X480;
      istwert=istwert-TOUCH_ADC_X0;
    #endif
    istwert=istwert*((float)(TOUCH_SIZE_X)/(float)(TOUCH_ADC_XDELTA));
    if(istwert>=TOUCH_SIZE_X) istwert=TOUCH_SIZE_X-1;
    Touch_Data.xp=(uint16_t)(istwert);
    // y-koordinate umrechnen
    istwert=y1;
    #if TOUCH_ADC_Y0>TOUCH_ADC_Y272
      #define   TOUCH_ADC_YDELTA      TOUCH_ADC_Y0-TOUCH_ADC_Y272
      if(istwert<TOUCH_ADC_Y272) istwert=TOUCH_ADC_Y272;
      if(istwert>TOUCH_ADC_Y0) istwert=TOUCH_ADC_Y0;
      istwert=TOUCH_ADC_Y0-istwert;
    #else
      #define   TOUCH_ADC_YDELTA      TOUCH_ADC_Y272-TOUCH_ADC_Y0
      if(istwert<TOUCH_ADC_Y0) istwert=TOUCH_ADC_Y0;
      if(istwert>TOUCH_ADC_Y272) istwert=TOUCH_ADC_Y272;
      istwert=istwert-TOUCH_ADC_Y0;
    #endif
    istwert=istwert*((float)(TOUCH_SIZE_Y)/(float)(TOUCH_ADC_YDELTA));
    if(istwert>=TOUCH_SIZE_Y) istwert=TOUCH_SIZE_Y-1;
    Touch_Data.yp=(uint16_t)(istwert);
    Touch_Data.status=TOUCH_PRESSED;
  }
  else {
    Touch_Data.status=TOUCH_RELEASED;
    Touch_Data.xp=0;
    Touch_Data.yp=0;
  }
}

//--------------------------------------------------------------
// interne Funktion
// ADC messen
// mode :     TOUCH_MODE_X = X-Achse
//            TOUCH_MODE_Y = Y-Achse
// ret_wert : 0 = ok
//            1 = error
//--------------------------------------------------------------
uint8_t P_Touch_480x272_ReadADC(uint8_t mode)
{
  uint8_t ret_wert=0;
  uint16_t istwert,min,max,n;
  
  if(mode==TOUCH_MODE_X) {
    // X-Achse
    // Messkanal einrichten
    ADC_RegularChannelConfig(TOUCH_ADC_NAME,TOUCH_ADC_CH_X,1,ADC_SampleTime_56Cycles);
    //-----------------------------------------------------
    // n messwerte einlesen und im Array speichern
    //-----------------------------------------------------
    TOUCH_RAW.x_adcwert=0;
    for(n=0;n<TOUCH_ADC_MESSUNGEN;n++) {
      do {
        // Start the conversion
        ADC_SoftwareStartConv(TOUCH_ADC_NAME);
        // Processing the conversion
        if (ADC_GetFlagStatus(TOUCH_ADC_NAME, ADC_FLAG_OVR)) ADC_ClearFlag(TOUCH_ADC_NAME, ADC_FLAG_OVR);       
      }
      while(ADC_GetFlagStatus(TOUCH_ADC_NAME,ADC_FLAG_EOC)==RESET);               
      // Messwert auslesen und speichern
      istwert=ADC_GetConversionValue(TOUCH_ADC_NAME);
      if(istwert>TOUCH_ADC_MAXWERT) istwert=TOUCH_ADC_MAXWERT;
      TOUCH_RAW.adc[n]=istwert;
    }
    // messwerte sortieren
    P_Touch_480x272_Sort();
    // messwerte auswerten
    min=TOUCH_RAW.adc[5];
    max=TOUCH_RAW.adc[TOUCH_ADC_MESSUNGEN-6];
    if((max-min)>TOUCH_ADC_MAXDELTA) ret_wert=1;
    istwert=TOUCH_RAW.adc[TOUCH_ADC_MESSUNGEN/2];
    if(istwert<TOUCH_ADC_MINXWERT) ret_wert=1;
    if(istwert>TOUCH_ADC_MAXXWERT) ret_wert=1;
    if(ret_wert==0) TOUCH_RAW.x_adcwert=istwert;
  }
  else if(mode==TOUCH_MODE_Y) {
    // Y-Achse
    // Messkanal einrichten
    ADC_RegularChannelConfig(TOUCH_ADC_NAME,TOUCH_ADC_CH_Y,1,ADC_SampleTime_56Cycles);
    //-----------------------------------------------------
    // n messwerte einlesen und im Array speichern
    //-----------------------------------------------------
    TOUCH_RAW.y_adcwert=0;
    for(n=0;n<TOUCH_ADC_MESSUNGEN;n++) {
      do {
        // Start the conversion
        ADC_SoftwareStartConv(TOUCH_ADC_NAME);
        // Processing the conversion
        if (ADC_GetFlagStatus(TOUCH_ADC_NAME, ADC_FLAG_OVR)) ADC_ClearFlag(TOUCH_ADC_NAME, ADC_FLAG_OVR);       
      }
      while(ADC_GetFlagStatus(TOUCH_ADC_NAME,ADC_FLAG_EOC)==RESET);               
      // Messwert auslesen und speichern
      istwert=ADC_GetConversionValue(TOUCH_ADC_NAME);
      if(istwert>TOUCH_ADC_MAXWERT) istwert=TOUCH_ADC_MAXWERT;
      TOUCH_RAW.adc[n]=istwert;
    }
    // messwerte sortieren
    P_Touch_480x272_Sort();
    // messwerte auswerten
    min=TOUCH_RAW.adc[5];
    max=TOUCH_RAW.adc[TOUCH_ADC_MESSUNGEN-6];
    if((max-min)>TOUCH_ADC_MAXDELTA) ret_wert=1;
    istwert=TOUCH_RAW.adc[TOUCH_ADC_MESSUNGEN/2];
    if(istwert<TOUCH_ADC_MINYWERT) ret_wert=1;
    if(istwert>TOUCH_ADC_MAXYWERT) ret_wert=1;
    if(ret_wert==0) TOUCH_RAW.y_adcwert=istwert;
  }
  
  return(ret_wert);
}

//--------------------------------------------------------------
// interne Funktion
// sortiert messwerte (bubblesort)
// (anfang=minimum, ende=maximum)
//--------------------------------------------------------------
void P_Touch_480x272_Sort(void)
{
  uint8_t m=0,stop=0;
  uint16_t a,b;
  uint16_t pos=0,max=(TOUCH_ADC_MESSUNGEN-1);

  do {
    a=TOUCH_RAW.adc[pos];
    b=TOUCH_RAW.adc[pos+1];
    if(b<a) {
      TOUCH_RAW.adc[pos]=b;
      TOUCH_RAW.adc[pos+1]=a;
      m=1;
    }
    pos++;
    if(pos>=max) {
      pos=0;
      if(m==0) stop=1;
      m=0;
      max--;
    }
  }while(stop==0);
}

//--------------------------------------------------------------
// interne Funktion
// kleine Pause
//-------------------------------------------------------------- 
void P_Touch_480x272_Delay(volatile uint32_t nCount)
{
  volatile uint32_t index = 0;

  for(index = nCount; index != 0; index--);
}

//--------------------------------------------------------------
// TIM ISR (1ms)
//--------------------------------------------------------------
void TOUCH_TIMER_ISR(void)
{
  if (TIM_GetITStatus(TOUCH_TIMER_NAME, TIM_IT_Update) != RESET) {
    // wenn Interrupt aufgetreten -> ISR flag lschen
    TIM_ClearITPendingBit(TOUCH_TIMER_NAME, TIM_IT_Update);

    TOUCH_RAW.timer_ms++;
    if(TOUCH_RAW.timer_ms>=TOUCH_INTERVALL_MS) {
      TOUCH_RAW.timer_ms=0;
      // touch messen
      P_Touch_480x272_ReadTouch();
    }
  }
}


